/**************************************************************/
/*                                                            */
/*   Program Name :  m5check                                  */
/*                                                            */
/*      Version:  1.51  2008-02-19                            */
/*                                                            */
/*  Copyright (c) 2003-2008 T.Kondo/NICT All Right Reserved   */
/*                                                            */
/**************************************************************/
/*

  Mark-5 data structure analysis

          T.Kondo 2003.03-11
                  2003.06-27 add non-parity 16bit-word mode
                  2003.10-13 add 64 track mode
                  2004.06-29 automatic both parity and non-parity modes check
                             becomes possible.
                             Results out was modified to display clearer
                  2004.09-20 head stack info was added
                  2004.09-25 default display mode was changed to monitor 
                               1st data block (8bytes) just after headder block
                  2004.10-20 debug in 1st data block pickup
                  2005.06-17 update due to the library restructuring
                  2006.09-26 add Video Converter # in output information
                  2008-01-08 debug for 2008 problem which is a problem 
                             of false detection of sync pattern of Mark5 
                             data time-labeled 2008            
                  2008-02-19 options '--help' and '--version'  were added 


    How to run
      m5check mk5file [mode] 
    
         where  mk5file -- Mark V data file name
                mode    -- data mode  (or display mode)
                             0: with parity and 8|16|32|64 bit-word mode 
                             1: without parity and 8|16|32|64 bit-word mode
                             2: to display using old display mode 
                             if mode is omitted, both modes are checked automatically
                             
G:\IPVLBI\data\jiveexp3\d03l1_2.on        --- 64 track non-parity
G:\IPVLBI\data\jiveexp2\d03c1_2.mc        --- 32 track non-parity
G:\IPVLBI\data\tsev5\1-177-1300.dat        --- 16 track non-parity
G:\mark5\8-301-1814.dat
G:\IPVLBI\data\evlbi6\11-K178-1321  --- VLBA format with parity converted from K5
G:\mark5\m5test.dat
G:\IPVLBI\data\GSIcheck\k08006_wz_006-0827  --- Mark5 data taken in 2008
*/

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libmk5tok5.h"  // library for mark5 to k5 conversion (by T.Kondo)

#define PROGRAM_VERSION "Ver 1.51  2008-02-19"


int main(int argc, char *argv[])
{
       int i,iret;
       //int header[160];  // header  5 X 32ch
       int header[320];  // header  5 X 64ch    2003-10-13
       int istart[6];
       int validtrk[64], trktable[64], hstack_tbl[64]; 
       int vcntabl[64];
       int ntrack;
       int mode,imode;
       int parity, nrzm, bitsframe;
       int dispmode;   // display mode 0: old style  1: new style
                                                      // 1 st data block display
       double fsampl;
       char mk5name[80];
       double telapsed;           // to measure processing time
       time_t tstart, tfinish;    //           "

       if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
          printf("\n");
          printf("m5check %s\n",PROGRAM_VERSION);
          printf("\n");
          printf("Usage: m5check m5file [mode]\n");
          printf("\n");
          printf("   where  m5file ---- Mark-5 data file name to cheak\n");
          printf("           mode ---- data mode\n");  
          printf("                     0: with parity 8|16|32|64 bit-word mode\n");
          printf("                     1: non parity  8|16|32|64 bit-word mode\n");
          printf("                     2: use old display style\n");
          printf("                     if mode is omitted, both modes are checked automatically\n");
          exit(0);
       }
       if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
          printf("\n");
          printf("m5check %s\n",PROGRAM_VERSION);
          printf("\n");
          exit(0);
       } 
        
       mode=-1;
       dispmode=1;

       /* data file name get */
       if (argc>=2){
          strcpy(mk5name,*(argv+1));
       }
       /* data mode get */
       if (argc>=3){
          mode= atoi(*(argv + 2 ));
          if(mode==2){
              dispmode=0;
              mode=-1;
          }
       }
       /* Program Name Display */
       printf("***********************************************************\n");
       printf("*    Mark-5 data structure analysis                       *\n");
       printf("*      %s  by  T.KONDO/NICT             *\n",PROGRAM_VERSION);
       printf("*                                                         *\n");
       if(mode==0){
       printf("*      NRZM, with parity, 8|16|32|64 bit-word mode        *\n");
       } else if(mode==1){
       printf("*      NRZL, non parity, 8|16|32|64 bit-word mode         *\n");
       } else {
       printf("*      automatic check both parity and non-parity modes   *\n");
       }
       printf("***********************************************************\n");
       printf("\n");

       tstart= time(NULL);

       /* Mark5 data structure check */

       if(mode==0 || mode ==1){
          if(mode==0){
             parity=1; nrzm=1;
             imode=0;
          } else {
             parity=0; nrzm=0;
             imode=1;
          }
          iret=mk5_dat_anal(mk5name, validtrk, trktable, hstack_tbl,
              header, &fsampl, istart, &ntrack, &bitsframe, parity, nrzm, dispmode,
              vcntabl);
       } else {
           parity=0; nrzm=0;
           for(i=0; i<2; i++){
              if(i==0){
                 imode=1;
                 printf("Non-parity mode is assumed first in data format analysis..\n");
              } 
              if(i==1){
                 imode=0;
                 parity=1; nrzm=1;
                 printf("No meaning data are found!\n");
                 printf("Parity mode is assumed in data format analysis..\n");
              }  
              if(mk5_dat_anal(mk5name, validtrk, trktable, hstack_tbl, header, &fsampl, 
                 istart, &ntrack, &bitsframe, parity, nrzm, dispmode,
                 vcntabl) >= 0) break; //  meaning data found
           }  
       } 
       if(ntrack ==0){
           printf("No meaning data are found!\n");
           return 0;
       }

       printf("\n");
       printf("**************** SUMMARY of DATA FORMAT ANALYSIS****************\n");
       printf(" File Name   : %s\n",mk5name);
       printf(" # of tracks : %d\n",ntrack);
       //printf(" # of bits (tracks) per word is %d\n",ntrack);
       if(imode==0){
           printf(" DATA mode   : NRZM  with parity\n");
       } else {
           printf(" DATA mode   : NRZL  without parity\n");
       }
       printf(" bits/frame  : %d ",bitsframe);
       if(bitsframe==20160) printf(" (This is VLBA Format)");
       if(bitsframe==20000) printf(" (This is Mark-IV Format)");
       printf("\n");
       if(istart[0]==-1){ // VLBA format
          printf(" 1st header time (MJD HH:MM:SS.SSSS) : %03d %02d:%02d:%02d.%04d\n",
           istart[1],istart[2],istart[3],istart[4],istart[5]);
       } else { // Mark-IV format
          printf(" 1st header time (Y/DDD HH:MM:SS.SSS) : %d/%03d %02d:%02d:%02d.%03d\n",
           istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);
       }
       printf(" Bit position and track# table : \n");
       printf("\n      bit pos  00 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15\n");
       printf("      Validity");
       for(i=0; i<16; i++) printf(" %2d",validtrk[i]);
       printf("\n");
       printf("      Track#  ");
       for(i=0; i<16; i++) printf(" %2d",trktable[i]);
       printf("\n");
       printf("      HDstack#");
       for(i=0; i<16; i++) printf(" %2d",hstack_tbl[i]);
       printf("\n");
       printf("      VideCon#");
       for(i=0; i<16; i++) printf(" %2d",vcntabl[i]);
       printf("\n");
       if(ntrack > 16){
       printf("\n      bit pos  16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31\n");
       printf("      Validity");
       for(i=16; i<32; i++) printf(" %2d",validtrk[i]);
       printf("\n");
       printf("      Track#  ");
       for(i=16; i<32; i++) printf(" %2d",trktable[i]);
       printf("\n");
       printf("      HDstack#");
       for(i=16; i<32; i++) printf(" %2d",hstack_tbl[i]);
       printf("\n");
       printf("      VideCon#");
       for(i=16; i<32; i++) printf(" %2d",vcntabl[i]);
       printf("\n");
       }
       if(ntrack > 32){
       printf("\n      bit pos  32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47\n");
       printf("      Validity");
       for(i=32; i<48; i++) printf(" %2d",validtrk[i]);
       printf("\n");
       printf("      Track#  ");
       for(i=32; i<48; i++) printf(" %2d",trktable[i]);
       printf("\n");
       printf("      HDstack#");
       for(i=32; i<48; i++) printf(" %2d",hstack_tbl[i]);
       printf("\n");
       printf("      VideCon#");
       for(i=32; i<48; i++) printf(" %2d",vcntabl[i]);
       printf("\n");
       printf("\n      bit pos  48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63\n");
       printf("      Validity");
       for(i=48; i<64; i++) printf(" %2d",validtrk[i]);
       printf("\n");
       printf("      Track#  ");
       for(i=48; i<64; i++) printf(" %2d",trktable[i]);
       printf("\n");
       printf("      HDstack#");
       for(i=48; i<64; i++) printf(" %2d",hstack_tbl[i]);
       printf("\n");
       printf("      VideCon#");
       for(i=48; i<64; i++) printf(" %2d",vcntabl[i]);
       printf("\n");
       }
       printf("\n");
       printf("****************************************************************\n");

 /*
       for(i=0; i<32; i++){
           printf("%02d ",i);
           for (k=0; k<5; k++){            
                 printf("%08x ",rvsint(header[5*i+k]));
           }
           printf("\n");
       }
*/
       tfinish= time(NULL);
       telapsed = difftime(tfinish,tstart);
    //   printf("Time elapsed (sec) for One obs process is %f\n",telapsed);


       return 0;
}
